@@ -4,18 +4,18 @@ import sys
 import transaction
 from BTrees.OOBTree import OOBTree
 from persistent import Persistent
-from zope.app.testing import setup
 from zope.app.folder import Folder
 from zope.app.container.sample import SampleContainer
 from zope.app.container.btree import BTreeContainer
 from zope.app.container.ordered import OrderedContainer
+from zope.app.testing import setup
 from zope.interface.verify import verifyObject
 from zope.interface import implements
-from zope.component import provideAdapter
 from zope.traversing.interfaces import IContainmentRoot
+from zope.component import provideAdapter
 
 from zodbbrowser.interfaces import IStateInterpreter
-from zodbbrowser.history import getHistory, loadState
+from zodbbrowser.history import ZodbObjectHistory
 from zodbbrowser.state import (GenericState,
                                EmptyOOBTreeState,
                                FolderState,
@@ -25,7 +25,8 @@ from zodbbrowser.state import (GenericState,
                                PersistentDictState,
                                PersistentMappingState,
                                SampleContainerState,
-                               FallbackState)
+                               FallbackState,
+                               ZodbObjectState, _loadState)
 from zodbbrowser.tests.realdb import RealDatabaseTest
 
 
@@ -41,6 +42,69 @@ class SampleFolder(Persistent, SampleContainer):
     pass
 
 
+class PersistentObject(Persistent):
+    pass # we need a subclass so we get a __dict__
+
+
+class TestZodbObjectState(RealDatabaseTest):
+
+    def setUp(self):
+        RealDatabaseTest.setUp(self)
+        provideAdapter(GenericState)
+        self.obj = self.conn.root()['obj'] = SampleFolder()
+        transaction.commit()
+
+    def testZodbObjectState(self):
+        state = ZodbObjectState(self.obj)
+        self.assertEquals(state.listItems(), None)
+        self.assertTrue(state.listAttributes()[0][0], '_SampleContainer__data')
+        self.assertEquals(state.getParent(), None)
+        self.assertEquals(state.getName(), '???')
+        self.assertTrue('_SampleContainer__data' in state.asDict().keys())
+
+
+class TestLoadState(RealDatabaseTest):
+
+    def setUp(self):
+        RealDatabaseTest.setUp(self)
+        root = self.conn.root()
+        self.adam = root['adam'] = PersistentObject()
+        transaction.commit()
+        self.eve = root['eve'] = PersistentObject()
+        transaction.commit()
+        self.adam.laptop = 'ThinkPad T23'
+        transaction.commit()
+        self.eve.laptop = 'MacBook'
+        transaction.commit()
+        self.adam.laptop = 'ThinkPad T42'
+        transaction.commit()
+        self.adam.laptop = 'ThinkPad T61'
+        transaction.commit()
+
+    def test_latest_state(self):
+        state = _loadState(self.adam).state
+        self.assertEquals(state, dict(laptop='ThinkPad T61'))
+
+    def test_exact_state(self):
+        tid = ZodbObjectHistory(self.adam)[1]['tid']
+        state = _loadState(self.adam, tid).state
+        self.assertEquals(state, dict(laptop='ThinkPad T42'))
+
+    def test_earlier_state(self):
+        tid = ZodbObjectHistory(self.eve)[0]['tid']
+        state = _loadState(self.adam, tid).state
+        self.assertEquals(state, dict(laptop='ThinkPad T23'))
+
+    def test_error_handling(self):
+        tid = ZodbObjectHistory(self.adam)[-1]['tid']
+        try:
+            _loadState(self.eve, tid).state
+        except Exception, e:
+            self.assertTrue("did not exist in or before" in str(e))
+        else:
+            self.fail("did not raise")
+
+
 class TestGenericState(unittest.TestCase):
 
     def test_interface_compliance(self):
@@ -109,7 +173,6 @@ class TestOrderedContainerState(RealDatabaseTest):
                                            None)
 
     def test_listItems(self):
-        items = self.state.listItems()
         self.assertEquals(list(self.state.listItems()),
                           [('foo', 1), ('bar', 2)])
 
@@ -117,7 +180,9 @@ class TestOrderedContainerState(RealDatabaseTest):
 class TestFolderState(RealDatabaseTest):
 
     def setUp(self):
+        setup.placelessSetUp()
         RealDatabaseTest.setUp(self)
+        provideAdapter(OOBTreeState)
         self.folder = self.conn.root()['folder'] = Folder()
         self.folder['foo'] = 1
         self.folder['bar'] = 2
@@ -125,8 +190,11 @@ class TestFolderState(RealDatabaseTest):
         self.state = FolderState(None, self.folder.__getstate__(),
                                  None)
 
+    def tearDown(self):
+        RealDatabaseTest.tearDown(self)
+        setup.placelessTearDown()
+
     def test_listItems(self):
-        items = self.state.listItems()
         self.assertEquals(list(self.state.listItems()),
                           [('bar', 2), ('foo', 1)])
 
@@ -138,7 +206,9 @@ class TestFolderState(RealDatabaseTest):
 class TestSampleContainerState(RealDatabaseTest):
 
     def setUp(self):
+        setup.placelessSetUp()
         RealDatabaseTest.setUp(self)
+        provideAdapter(OOBTreeState)
         self.container = self.conn.root()['container'] = BTreeContainer()
         self.container['foo'] = 1
         self.container['bar'] = 2
@@ -146,8 +216,11 @@ class TestSampleContainerState(RealDatabaseTest):
         self.state = SampleContainerState(None, self.container.__getstate__(),
                                           None)
 
+    def tearDown(self):
+        RealDatabaseTest.tearDown(self)
+        setup.placelessTearDown()
+
     def test_listItems(self):
-        items = self.state.listItems()
         self.assertEquals(list(self.state.listItems()),
                           [('bar', 2), ('foo', 1)])
 
@@ -160,7 +233,9 @@ class TestSampleContainerState(RealDatabaseTest):
 class TestBTreeContainerState(RealDatabaseTest):
 
     def setUp(self):
+        setup.placelessSetUp()
         RealDatabaseTest.setUp(self)
+        provideAdapter(OOBTreeState)
         self.container = self.conn.root()['container'] = BTreeContainer()
         self.container['foo'] = 1
         self.container['bar'] = 2
@@ -168,8 +243,11 @@ class TestBTreeContainerState(RealDatabaseTest):
         self.state = BTreeContainerState(None, self.container.__getstate__(),
                                          None)
 
+    def tearDown(self):
+        RealDatabaseTest.tearDown(self)
+        setup.placelessTearDown()
+
     def test_listItems(self):
-        items = self.state.listItems()
         self.assertEquals(list(self.state.listItems()),
                           [('bar', 2), ('foo', 1)])
 
@@ -220,10 +298,10 @@ class TestLargeOOBTreeState(RealDatabaseTest):
         for i in range(0, 1000, 2):
             self.tree[i] = 1
         transaction.commit()
-        self.tids = [d['tid'] for d in getHistory(self.tree)]
+        self.tids = [d['tid'] for d in ZodbObjectHistory(self.tree)]
 
     def getState(self, tid):
-        state = loadState(self.tree, tid)
+        state = _loadState(self.tree, tid).state
         return OOBTreeState(self.tree, state, tid)
 
     def test_current_state(self):
